//////////////////////////////////////////////////////
// Kinematics files for Robotics applications
// 
// Copyright (c) 2002-2010. All Rights Reserved.
// Division of Applied Robot Technology, KITECH
// Web: http://www.orobot.net
// Written by KwangWoong Yang<page365@gmail.com>
//

#pragma once
#include "Transformation.h"
#include "MathDef.h"
#include <string>

using namespace std;

enum eObjectType { 
	TRANSFORM_PUSH,
	TRANSFORM_POP,
	FIXED_JOINT,		//  Ʈ: ǥ ̵ϱ Ѵ.
	REVOLUTE_JOINT,		// ȸ  Ʈ
	PRISMATIC_JOINT,	//   Ʈ
	SOLID_LINE,
	SOLID_SPHERE,
	SOLID_CAPSULE,
	SOLID_CYLINDER,
	SOLID_BOX,
};

class ObjectBase
{
public:
	ObjectBase(const ObjectBase &o) : 
		_type (o._type), _name(o._name), _color(o._color), 
		_x(o._x), _y(o._y), _z(o._z), _phi(o._phi), _theta(o._theta), _psi(o._psi) { }
	ObjectBase(eObjectType type, const char *name, unsigned long color, 
		double x, double y, double z, double phi, double theta, double psi) :
		_type (type), _name(name), _color(color), 
		_x(x), _y(y), _z(z), _phi(phi), _theta(theta), _psi(psi) { }
	virtual ~ObjectBase(void) { }

	virtual CTransformMatrix TransformationMatrix ();

public:
	eObjectType _type;
	string _name;
	unsigned long _color;

	double _x, _y, _z;
	double _phi, _theta, _psi;
};

class ObjectJoint : public ObjectBase
{
public:
	ObjectJoint (const ObjectBase &o, int axis, double radius, 
		double limit_lo = -M_PI, double limit_hi = M_PI, double median = 0., double weight = 1.) : 
		ObjectBase(o), _axis(axis), _radius(radius), 
		_limit_lo(-limit_lo), _limit_hi(limit_hi), _median(median), _weight(weight), 
		_q(0.), a(3), p(3)
	{ 
		assert (0 <= _axis && _axis < 3);
	}

	CTransformMatrix TransformationMatrixQ ();
	CTransformMatrix TransformationMatrix ();

public:
	// Ʈ ʱ ġ
	int _axis;					// ȸ()  ߽   (0 - x, 1 - y, 2 - z)
	double _radius;				// ũ , OpenGL ׸ ؼ 

	// α׷   Ǵ 
	double _q;					// Ʈ  ȸ() ġ
	double _limit_lo, _limit_hi;	// Ʈ   (limit_lo ~ limit_hi)
	double _median;				// Ʈ ߰  (Null motion Ҵϱ  )
	double _weight;				// Ʈ ġ

	dVector a;					// ȸ  ӵ
	dVector p;					//  ǥ迡 ȸ  ġ
};

class ObjectLine : public ObjectBase
{
public:
	ObjectLine (const ObjectBase &o, double length) : 
		ObjectBase(o), _length(length) { }

public:
	double _length;
};

class ObjectSphere : public ObjectBase
{
public:
	ObjectSphere (const ObjectBase &o, double radius) : 
		ObjectBase(o), _radius(radius) { }

public:
	double _radius;
};

class ObjectCapsule : public ObjectBase
{
public:
	ObjectCapsule (const ObjectBase &o, double radius, double height) : 
		ObjectBase(o), _radius(radius), _height(height) { }

public:
	double _radius, _height;
};

class ObjectCylinder : public ObjectBase
{
public:
	ObjectCylinder (const ObjectBase &o, double radius, double height) : 
		ObjectBase(o), _radius(radius), _height(height) { }

public:
	double _radius, _height;
};

class ObjectBox : public ObjectBase
{
public:
	ObjectBox (const ObjectBase &o, double dx, double dy, double dz) : 
		ObjectBase(o), _dx(dx), _dy(dy), _dz(dz) { }

public:
	double _dx, _dy, _dz;
};

