//////////////////////////////////////////////////////
// OpenGL files for Robotics applications
// 
// Copyright (c) 2002-2010. All Rights Reserved.
// Division of Applied Robot Technology, KITECH
// Web: http://www.orobot.net
// Written by KwangWoong Yang<page365@gmail.com>
//

#include "StdAfx.h"
#include "OglWndCustom.h"
#include "MathDef.h"
#include "Transformation.h"
#include "OglObjects.h"
#include <gl/glu.h>
#include "Objects.h"

COglWndCustom::COglWndCustom (CObjectContainer *objs)
: _objs(objs), _showAxis(false)
{
}

// Rendering
void COglWndCustom::OnDraw (CDC* pDC)
{
	SetViewport ();
	oglPlane (5., .5);

	RenderJoint ();
}

void COglWndCustom::TransformAxis(dMatrix &A)
{
	double m[16];

	m[0] = A(0,0);  m[4] = A(0,1);  m[8] = A(0,2);  m[12] = A(0,3); 
	m[1] = A(1,0);  m[5] = A(1,1);  m[9] = A(1,2);  m[13] = A(1,3); 
	m[2] = A(2,0);  m[6] = A(2,1);  m[10]= A(2,2);  m[14] = A(2,3); 
	m[3] = 0.;      m[7] = 0.;      m[11]= 0.;      m[15] = 1.; 

	glMultMatrixd(m);
}

void COglWndCustom::DrawRevLink (double x, double y, double z, double radius)
{
	glPushMatrix();

	double rz = _RAD2DEG*atan2(y, x);
	double ry = 90. - _RAD2DEG*atan2(z, sqrt(x*x + y*y));
	double height = sqrt (x*x + y*y + z*z);

	glRotated( rz, 0, 0, 1);
	glRotated( ry, 0, 1, 0);
	glTranslated (0, 0, height/2.);		

	oglCylinder (height, radius);

	glPopMatrix();
}

void COglWndCustom::DrawFixedJoint(ObjectJoint *joint)
{
	DrawRevLink (joint->_x, joint->_y, joint->_z, joint->_radius);
}

void COglWndCustom::DrawRevoluteJoint(ObjectJoint *joint)
{
	glPushMatrix();

	if      (joint->_axis == 0) glRotated( 90., 0,1,0 );
	else if (joint->_axis == 1) glRotated(-90., 1,0,0 );

	glColor3d(0.8*GetRValue (joint->_color)/255., 0.8*GetGValue (joint->_color)/255., 0.8*GetBValue (joint->_color)/255.);
	oglCylinder (joint->_radius*2.5, joint->_radius*1.3);
	glColor3d(GetRValue (joint->_color)/255., GetGValue (joint->_color)/255., GetBValue (joint->_color)/255.);

	glPopMatrix();

	glPushMatrix();

	TransformAxis (joint->TransformationMatrixQ ());
	DrawRevLink (joint->_x, joint->_y, joint->_z, joint->_radius);

	glPopMatrix();
}

void COglWndCustom::DrawPrismaticJoint(ObjectJoint *joint)
{
	glPushMatrix();

	if      (joint->_axis == 0) glRotated( 90., 0,1,0 );
	else if (joint->_axis == 1) glRotated(-90., 1,0,0 );

	oglBox (joint->_radius*2.5, joint->_radius*2.5, joint->_radius*2.5);
	glTranslated (0, 0, joint->_q/2.);		

	glColor3d(0.8*GetRValue (joint->_color)/255., 0.8*GetGValue (joint->_color)/255., 0.8*GetBValue (joint->_color)/255.);
	oglBox (joint->_radius*1.5, joint->_radius*1.5, joint->_q);
	glColor3d(GetRValue (joint->_color)/255., GetGValue (joint->_color)/255., GetBValue (joint->_color)/255.);

	glPopMatrix();

	glPushMatrix();

	TransformAxis (joint->TransformationMatrixQ ());
	DrawRevLink (joint->_x, joint->_y, joint->_z, joint->_radius);

	glPopMatrix();
}

void COglWndCustom::RenderJoint()
{
	vector<ObjectBase *> &ol = _objs->GetObjectList ();

	for (vector<ObjectBase *>::iterator it = ol.begin (); it != ol.end (); ++it) {
		ObjectBase *o = *it;

		glColor3d(GetRValue (o->_color)/255., GetGValue (o->_color)/255., GetBValue (o->_color)/255.);

		switch (o->_type) {
		case TRANSFORM_PUSH:	glPushMatrix();	break;
		case TRANSFORM_POP:		glPopMatrix();  break;
		case FIXED_JOINT:		DrawFixedJoint (dynamic_cast<ObjectJoint *>(o));		TransformAxis (o->TransformationMatrix ());		break;
		case REVOLUTE_JOINT:	DrawRevoluteJoint (dynamic_cast<ObjectJoint *>(o));		TransformAxis (o->TransformationMatrix ());		break;
		case PRISMATIC_JOINT:	DrawPrismaticJoint (dynamic_cast<ObjectJoint *>(o));	TransformAxis (o->TransformationMatrix ());		break;
		case SOLID_LINE:		TransformAxis (o->TransformationMatrix ());		DrawLine (dynamic_cast<ObjectLine *>(o));			break;	
		case SOLID_SPHERE:		TransformAxis (o->TransformationMatrix ());		DrawSphere (dynamic_cast<ObjectSphere *>(o));		break;	
		case SOLID_CAPSULE:		TransformAxis (o->TransformationMatrix ());		DrawCapsule (dynamic_cast<ObjectCapsule *>(o));		break;	
		case SOLID_CYLINDER:	TransformAxis (o->TransformationMatrix ());		DrawCylinder (dynamic_cast<ObjectCylinder *>(o));	break;	
		case SOLID_BOX:			TransformAxis (o->TransformationMatrix ());		DrawBox (dynamic_cast<ObjectBox *>(o));				break;	
		}
	}
	// if (_showAxis) oglCoordinate (0.3);
}


